( cd usr/bin ; rm -rf wish )
( cd usr/bin ; ln -sf wish8.4 wish )
( cd usr/lib ; rm -rf libtk.so )
( cd usr/lib ; ln -sf libtk8.4.so libtk.so )
( cd usr/man/man3 ; rm -rf Tk_UnmaintainGeometry.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MaintainGeometry.3.gz Tk_UnmaintainGeometry.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameToWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Name.3.gz Tk_NameToWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasGetCoord.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasGetCoord.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteErrorHandler.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateErrorHandler.3.gz Tk_DeleteErrorHandler.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetFont.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocFontFromObj.3.gz Tk_GetFont.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ResizeWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_ResizeWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoSetSize.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoSetSize.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DistanceToTextLayout.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_DistanceToTextLayout.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfColor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_NameOfColor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_TextLayoutToPostscript.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_TextLayoutToPostscript.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_GetBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetCursorFromData.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_GetCursorFromData.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetCursorFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_GetCursorFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Offset.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_Offset.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasPsFont.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasPsY.3.gz Tk_CanvasPsFont.3.gz )
( cd usr/man/man3 ; rm -rf Tk_IsMapped.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_IsMapped.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetOptionInfo.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_GetOptionInfo.3.gz )
( cd usr/man/man3 ; rm -rf Tk_3DHorizontalBevel.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_3DHorizontalBevel.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfAnchor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetAnchorFromObj.3.gz Tk_NameOfAnchor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PostscriptFontName.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FontId.3.gz Tk_PostscriptFontName.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeColorFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_FreeColorFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetRelief.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetReliefFromObj.3.gz Tk_GetRelief.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetFontFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocFontFromObj.3.gz Tk_GetFontFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetColorFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_GetColorFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SafeInit.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Init.3.gz Tk_SafeInit.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetImageMasterData.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateImageType.3.gz Tk_GetImageMasterData.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Parent.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Parent.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeImage.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetImage.3.gz Tk_FreeImage.3.gz )
( cd usr/man/man3 ; rm -rf Tk_BindEvent.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_BindEvent.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasSetStippleOrigin.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasSetStippleOrigin.3.gz )
( cd usr/man/man3 ; rm -rf Tk_IsContainer.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_IsContainer.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetNumMainWindows.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MainWindow.3.gz Tk_GetNumMainWindows.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetMinimumRequestSize.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GeometryRequest.3.gz Tk_SetMinimumRequestSize.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasPsPath.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasPsY.3.gz Tk_CanvasPsPath.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeFontFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocFontFromObj.3.gz Tk_FreeFontFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Draw3DRectangle.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Draw3DRectangle.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetBinding.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_GetBinding.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ReqHeight.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_ReqHeight.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeCursor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_FreeCursor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoGetSize.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoGetSize.3.gz )
( cd usr/man/man3 ; rm -rf Tk_RestoreSavedOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_RestoreSavedOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfFont.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocFontFromObj.3.gz Tk_NameOfFont.3.gz )
( cd usr/man/man3 ; rm -rf Tk_UnmapWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MapWindow.3.gz Tk_UnmapWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_UndefineCursor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_UndefineCursor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_NameOfBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Attributes.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Attributes.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasPsColor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasPsY.3.gz Tk_CanvasPsColor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreePixmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetPixmap.3.gz Tk_FreePixmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteOptionTable.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_DeleteOptionTable.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasPsStipple.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasPsY.3.gz Tk_CanvasPsStipple.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetInternalBorderEx.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GeometryRequest.3.gz Tk_SetInternalBorderEx.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeFont.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocFontFromObj.3.gz Tk_FreeFont.3.gz )
( cd usr/man/man3 ; rm -rf Tk_RedrawImage.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetImage.3.gz Tk_RedrawImage.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowBorderPixmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowBorderPixmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetOptionValue.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_GetOptionValue.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PointToChar.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_PointToChar.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InitImageArgs.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateImageType.3.gz Tk_InitImageArgs.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Class.3.gz )
( cd usr/man/man3 ; ln -sf Tk_SetClass.3.gz Tk_Class.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CreateBinding.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_CreateBinding.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfRelief.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetReliefFromObj.3.gz Tk_NameOfRelief.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SizeOfImage.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetImage.3.gz Tk_SizeOfImage.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetPixels.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetPixelsFromObj.3.gz Tk_GetPixels.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetBackgroundFromBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_SetBackgroundFromBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeConfigOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_FreeConfigOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasDrawableCoords.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasDrawableCoords.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ConfigureInfo.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWidget.3.gz Tk_ConfigureInfo.3.gz )
( cd usr/man/man3 ; rm -rf Tk_IsEmbedded.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_IsEmbedded.3.gz )
( cd usr/man/man3 ; rm -rf Tk_MoveResizeWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_MoveResizeWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_3DBorderColor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_3DBorderColor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoPutBlock.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoPutBlock.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Get3DBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Get3DBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteClientMessageHandler.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateClientMessageHandler.3.gz Tk_DeleteClientMessageHandler.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetInternalBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GeometryRequest.3.gz Tk_SetInternalBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasEventuallyRedraw.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasEventuallyRedraw.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteGenericHandler.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateGenericHandler.3.gz Tk_DeleteGenericHandler.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetJustify.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetJustifyFromObj.3.gz Tk_GetJustify.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetAtomName.3.gz )
( cd usr/man/man3 ; ln -sf Tk_InternAtom.3.gz Tk_GetAtomName.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DefineBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_DefineBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeGC.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetGC.3.gz Tk_FreeGC.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoGetImage.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoGetImage.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWidget.3.gz Tk_FreeOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetCursor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_GetCursor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowBorderWidth.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowBorderWidth.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOf3DBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_NameOf3DBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Width.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Width.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ConfigureValue.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWidget.3.gz Tk_ConfigureValue.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Fill3DPolygon.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Fill3DPolygon.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Ungrab.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Grab.3.gz Tk_Ungrab.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CharBbox.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_CharBbox.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteBindingTable.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_DeleteBindingTable.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasTagsOption.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasTagsOption.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Screen.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Screen.3.gz )
( cd usr/man/man3 ; rm -rf Tk_MinReqWidth.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_MinReqWidth.3.gz )
( cd usr/man/man3 ; rm -rf Tk_3DVerticalBevel.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_3DVerticalBevel.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Height.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Height.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteSelHandler.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateSelHandler.3.gz Tk_DeleteSelHandler.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InternalBorderRight.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_InternalBorderRight.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeBitmapFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_FreeBitmapFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_SetOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetScreenMM.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetPixelsFromObj.3.gz Tk_GetScreenMM.3.gz )
( cd usr/man/man3 ; rm -rf Tk_MakeWindowExist.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateWindow.3.gz Tk_MakeWindowExist.3.gz )
( cd usr/man/man3 ; rm -rf Tk_IsTopLevel.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_IsTopLevel.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Colormap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Colormap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_QueueWindowEvent.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CollapseMotionEvents.3.gz Tk_QueueWindowEvent.3.gz )
( cd usr/man/man3 ; rm -rf Tk_AttachHWND.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetHWND.3.gz Tk_AttachHWND.3.gz )
( cd usr/man/man3 ; rm -rf Tk_UnderlineTextLayout.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_UnderlineTextLayout.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfJoinStyle.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetJoinStyle.3.gz Tk_NameOfJoinStyle.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfCapStyle.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetCapStyle.3.gz Tk_NameOfCapStyle.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Fill3DRectangle.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Fill3DRectangle.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteAllBindings.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_DeleteAllBindings.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Draw3DPolygon.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Draw3DPolygon.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DrawTextLayout.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_DrawTextLayout.3.gz )
( cd usr/man/man3 ; rm -rf Tk_MoveWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_MoveWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ReqWidth.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_ReqWidth.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SizeOfBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_SizeOfBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ClipboardAppend.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ClipboardClear.3.gz Tk_ClipboardAppend.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoBlank.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoBlank.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Depth.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Depth.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfCursor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_NameOfCursor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Free3DBorder.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Free3DBorder.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InternalBorderTop.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_InternalBorderTop.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteBinding.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_DeleteBinding.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InternalBorderLeft.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_InternalBorderLeft.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeCursorFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocCursorFromObj.3.gz Tk_FreeCursorFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowBackground.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowBackground.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Uid.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetUid.3.gz Tk_Uid.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeColor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_FreeColor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetFontMetrics.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FontId.3.gz Tk_GetFontMetrics.3.gz )
( cd usr/man/man3 ; rm -rf Tk_X.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_X.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetColor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_GetColor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Y.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Y.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DestroyWindow.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateWindow.3.gz Tk_DestroyWindow.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeSavedOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_FreeSavedOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_TextWidth.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MeasureChars.3.gz Tk_TextWidth.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasPsBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasPsY.3.gz Tk_CanvasPsBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InternalBorderBottom.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_InternalBorderBottom.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoExpand.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoExpand.3.gz )
( cd usr/man/man3 ; rm -rf Tk_MinReqHeight.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_MinReqHeight.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeColormap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetColormap.3.gz Tk_FreeColormap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Changes.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Changes.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PathName.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Name.3.gz Tk_PathName.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CreateWindowFromPath.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateWindow.3.gz Tk_CreateWindowFromPath.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeBitmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_FreeBitmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Visual.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Visual.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DisplayName.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_DisplayName.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DrawChars.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MeasureChars.3.gz Tk_DrawChars.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DefineCursor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_DefineCursor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_FreeTextLayout.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_FreeTextLayout.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Free3DBorderFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Free3DBorderFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetAllBindings.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateBindingTable.3.gz Tk_GetAllBindings.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetAnchor.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetAnchorFromObj.3.gz Tk_GetAnchor.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetBitmapFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocBitmapFromObj.3.gz Tk_GetBitmapFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_UnderlineChars.3.gz )
( cd usr/man/man3 ; ln -sf Tk_MeasureChars.3.gz Tk_UnderlineChars.3.gz )
( cd usr/man/man3 ; rm -rf Tk_NameOfJustify.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetJustifyFromObj.3.gz Tk_NameOfJustify.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ScreenNumber.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_ScreenNumber.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetItemTypes.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateItemType.3.gz Tk_GetItemTypes.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowColormap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowColormap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_InitOptions.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateOptionTable.3.gz Tk_InitOptions.3.gz )
( cd usr/man/man3 ; rm -rf Tk_CanvasWindowCoords.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CanvasTkwin.3.gz Tk_CanvasWindowCoords.3.gz )
( cd usr/man/man3 ; rm -rf Tk_IntersectTextLayout.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ComputeTextLayout.3.gz Tk_IntersectTextLayout.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Display.3.gz )
( cd usr/man/man3 ; ln -sf Tk_WindowId.3.gz Tk_Display.3.gz )
( cd usr/man/man3 ; rm -rf Tk_ChangeWindowAttributes.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_ChangeWindowAttributes.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetMMFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetPixelsFromObj.3.gz Tk_GetMMFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_UnsetGrid.3.gz )
( cd usr/man/man3 ; ln -sf Tk_SetGrid.3.gz Tk_UnsetGrid.3.gz )
( cd usr/man/man3 ; rm -rf Tk_PhotoPutZoomedBlock.3.gz )
( cd usr/man/man3 ; ln -sf Tk_FindPhoto.3.gz Tk_PhotoPutZoomedBlock.3.gz )
( cd usr/man/man3 ; rm -rf Tk_SetWindowBackgroundPixmap.3.gz )
( cd usr/man/man3 ; ln -sf Tk_ConfigureWindow.3.gz Tk_SetWindowBackgroundPixmap.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetColorByValue.3.gz )
( cd usr/man/man3 ; ln -sf Tk_AllocColorFromObj.3.gz Tk_GetColorByValue.3.gz )
( cd usr/man/man3 ; rm -rf Tk_Get3DBorderFromObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_Get3DBorderFromObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_GetScrollInfoObj.3.gz )
( cd usr/man/man3 ; ln -sf Tk_GetScrollInfo.3.gz Tk_GetScrollInfoObj.3.gz )
( cd usr/man/man3 ; rm -rf Tk_3DBorderGC.3.gz )
( cd usr/man/man3 ; ln -sf Tk_Alloc3DBorderFromObj.3.gz Tk_3DBorderGC.3.gz )
( cd usr/man/man3 ; rm -rf Tk_DeleteEventHandler.3.gz )
( cd usr/man/man3 ; ln -sf Tk_CreateEventHandler.3.gz Tk_DeleteEventHandler.3.gz )
( cd usr/man/mann ; rm -rf tk_textCut.n.gz )
( cd usr/man/mann ; ln -sf text.n.gz tk_textCut.n.gz )
( cd usr/man/mann ; rm -rf tk_focusFollowsMouse.n.gz )
( cd usr/man/mann ; ln -sf tk_focusNext.n.gz tk_focusFollowsMouse.n.gz )
( cd usr/man/mann ; rm -rf tk_bisque.n.gz )
( cd usr/man/mann ; ln -sf tk_setPalette.n.gz tk_bisque.n.gz )
( cd usr/man/mann ; rm -rf tk_bindForTraversal.n.gz )
( cd usr/man/mann ; ln -sf tk_menuBar.n.gz tk_bindForTraversal.n.gz )
( cd usr/man/mann ; rm -rf tk_focusPrev.n.gz )
( cd usr/man/mann ; ln -sf tk_focusNext.n.gz tk_focusPrev.n.gz )
( cd usr/man/mann ; rm -rf tk_menuSetFocus.n.gz )
( cd usr/man/mann ; ln -sf menu.n.gz tk_menuSetFocus.n.gz )
( cd usr/man/mann ; rm -rf tk_textPaste.n.gz )
( cd usr/man/mann ; ln -sf text.n.gz tk_textPaste.n.gz )
( cd usr/man/mann ; rm -rf tk_getSaveFile.n.gz )
( cd usr/man/mann ; ln -sf tk_getOpenFile.n.gz tk_getSaveFile.n.gz )
( cd usr/man/mann ; rm -rf tk_textCopy.n.gz )
( cd usr/man/mann ; ln -sf text.n.gz tk_textCopy.n.gz )
